# ---------------------------------------------------------------
# Copyright (C) 2011 Merrimack Valley Library Consortium
# Jason Stephenson <jstephenson@mvlc.org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ---------------------------------------------------------------
#
# An object to handle fee payment
#

package OpenILS::SIP::Transaction::FeePayment;

use warnings;
use strict;

use POSIX qw(strftime);

use OpenILS::SIP;
use OpenILS::SIP::Transaction;
use OpenILS::SIP::Msg qw/:const/;
use Sys::Syslog qw(syslog);

use OpenILS::Application::AppUtils;
my $U = 'OpenILS::Application::AppUtils';


our @ISA = qw(OpenILS::SIP::Transaction);

# Most fields are handled by the Transaction superclass
my %fields = (
    sip_payment_type => undef,
    fee_id => 0,
    patron_password => undef,
    );

sub new {
    my $class = shift;

    my $self = $class->SUPER::new(@_);

    foreach my $element (keys %fields) {
        $self->{_permitted}->{$element} = $fields{$element};
    }

    @{$self}{keys %fields} = values %fields;
    return bless $self, $class;
}

sub do_fee_payment {
    my $self = shift;
    my $results = $U->simplereq('open-ils.actor', 'open-ils.actor.user.transactions.history.have_balance', $self->{authtoken}, $self->patron->internal_id);
    if (ref $results eq 'ARRAY') {
        syslog('LOG_INFO', scalar @$results . " bills found for " . $self->patron->internal_id);
        my $payment = $self->fee_amount;
        foreach my $bill (@{$results}) {
            syslog('LOG_INFO', 'bill '. $bill->id . ' amount ' . $bill->balance_owed);
            if (!$self->transaction_id) {
                if ($bill->balance_owed >= $payment) {
                    $self->pay_bill($bill->id, $payment);
                    $payment = 0;
                }
                else {
                    $self->pay_bill($bill->id, $bill->balance_owed);
                    $payment -= $bill->balance_owed;
                }
                last unless ($payment);
            }
            elsif ($self->transaction_id == $bill->id) {
                $self->pay_bill($bill->id, $payment);
                $payment = 0;
                last;
            }
        }
        if ($payment == $self->fee_amount) {
            $self->ok(0);
            if (!$self->transaction_id) {
                $self->screen_msg("Payment failed: you have no bills.");
            }
            else {
                $self->screen_msg("Payment failed: transaction " + $self->transaction_id + " not found.");
            }
        }
        else {
            $self->ok(1);
        }
    }
    else {
        $self->ok(0);
        $self->screen_msg($results->{descr});
    }
}

sub pay_bill {
    my ($self, $bill, $amount) = @_;
    my $user = $self->patron->{user};
    my $r = $U->simplereq('open-ils.circ', 'open-ils.circ.money.payment', $self->{authtoken},
                          { payment_type => "cash_payment", userid => $user->id, note => "via SIP2",
                            payments => [[$bill, $amount ]]}, $user->last_xact_id);
}


1;
